/* See aagraph.h */
/* Last edited on 2005-09-25 18:46:07 by stolfi */

#include "aagraph.h"
#include <affirm.h>
#include <flt.h>
#include <ia.h>
#include <aa.h>
#include <aarange.h>
#include <ps.h>
#include <math.h>
#include <stdio.h>

void aagraph_plot_paralelograms(
    FILE *psfile,
    AAP f (AAP x),
    Interval xd,
    Interval yd,
    int n
  )
  {
    Interval xv, yvlo, yvhi;
    int xi;
    AAP xf, yf;
    double xp[4], yp[4];
    AATerm eps[1];
    double gray = 0.75;

    ps_comment(psfile, "Plot of AA graph");

    for (xi=0; xi<n; xi++)
      {
        MemP frame = aa_top();

        ROUND_DOWN;
        xv.lo = xd.lo + ((xd.hi - xd.lo)*xi)/n;

        ROUND_UP;
        xv.hi = xd.lo + ((xd.hi - xd.lo)*(xi+1))/n;

        xf = aa_from_interval(xv);
        affirm((xf->nterms == 1), "aagraph_plot_paralelograms: xf->nterms != 1");
        eps[0].id = ((AATermP)(xf + 1))->id;

        yf = f(xf);
        
        eps[0].coef = -One;
        yvlo = aa_range(aa_fix_eps(yf, 1, eps));
        eps[0].coef = One;
        yvhi = aa_range(aa_fix_eps(yf, 1, eps));

        if (ia_is_full(&yvlo) || ia_is_full(&yvhi)) 
          { yvlo = (Interval){xd.lo, xd.hi}; yvhi = yvlo; }

        ROUND_NEAR;
        xp[0] = xv.lo;  yp[0] = yvlo.lo;
        xp[1] = xv.hi;  yp[1] = yvhi.lo;
        xp[2] = xv.hi;  yp[2] = yvhi.hi;
        xp[3] = xv.lo;  yp[3] = yvlo.hi;
        
        ps_fill_and_draw_polygon(psfile, xp, yp, 4, gray,gray,gray);

        aa_flush(frame);
      }

    fprintf(stderr, "\n");
  }

void aagraph_plot_boxes(
    FILE *psfile,
    AAP f (AAP x),
    Interval xd,
    Interval yd,
    int n
  )
  {
    Interval xv, yv;
    int xi;
    AAP xf, yf;
    double gray = 0.75;

    ps_comment(psfile, "Plot of AA range graph");

    for (xi=0; xi<n; xi++)
      {
        MemP frame = aa_top();

        ROUND_DOWN;
        xv.lo = xd.lo + ((xd.hi - xd.lo)*xi)/n;

        ROUND_UP;
        xv.hi = xd.lo + ((xd.hi - xd.lo)*(xi+1))/n;

        xf = aa_from_interval(xv);

        yf = f(xf);
        yv = aa_range(yf);

        if (ia_is_full(&yv)) { yv = (Interval){xd.lo, xd.hi}; }

        ROUND_NEAR;
        ps_fill_and_draw_rectangle(psfile, xv.lo, xv.hi, yv.lo, yv.hi, gray,gray,gray);

        aa_flush(frame);
      }

    fprintf(stderr, "\n");
  }

void aagraph_fill_and_draw_2d_range 
  ( FILE *psfile, 
    AAP x, 
    AAP y, 
    double R, double G, double B
  )
  {
    AATermCount xn = (x->nterms);
    AATermCount yn = (y->nterms);
    AATermCount nvmax = 2*(xn+yn);
    double xv[nvmax], yv[nvmax];
    AATermCount nv;

    ps_comment(psfile, "AA joint range");

    aa_2d_range(x, y, &nv, xv, yv);
    if (nv == 0)
      { ps_draw_dot(psfile, (double)(x->center), (double)(y->center), 0.5); }
    else
      { ps_fill_and_draw_polygon(psfile, xv, yv, nv, R,G,B); }
    
    fprintf(stderr, "\n");
  }
