/* Timing of floating point ops (from flt.h) */
/* Last edited on 2007-01-04 00:14:38 by stolfi */

#include <flt.h>
#include <ioprotos.h>
#include <stdio.h>
#include <timefunc.h>

// #if (defined(SunOS4))
// #  include <sys/ieeefp.h>
// int ieee_flags(char *a, char *b, char *c, char **out);
// #endif
// 
// #if (defined(SunOS5))
// #  include <ieeefp.h>
// #endif

#include <fenv.h>
#include <fpu_control.h>

// #if (defined(OSF1V4))
// /* #  include <ieeefp.h> */
// #endif

int main (int argc, char **argv);

int do_empty(int ntimes);
int do_conversion(int ntimes);
int do_single_add(int ntimes);
int do_single_mul(int ntimes);
int do_single_div(int ntimes);
int do_double_add(int ntimes);
int do_double_mul(int ntimes);
int do_double_div(int ntimes);
int do_my_set_round(int ntimes);
int do_sys_set_round(int ntimes);
int do_flt_mul(int ntimes);
int do_flt_mix(int ntimes);
int do_FMIN_FMAX(int ntimes);

#define NTIMES 100000000

int main (int argc, char **argv)
  {
    flt_init();
    
    { /* Tests flt_mul overflow: */
      Float x, y, z, err;
      x = MaxFloat/2.0;
      y = x;
      err = 0;
      flt_mul(x, y, &z, &err);
      ROUND_NEAR;
      fprintf(stderr, " x =    %18.8f\n", x);
      fprintf(stderr, " y =    %18.8f\n", y);
      fprintf(stderr, " z =    %18.8f\n", z);
      fprintf(stderr, " err =  %18.8f\n", err);
      fprintf(stderr, " +Inf = %18.8f\n", PlusInfinity);
      fprintf(stderr, " -Inf = %18.8f\n", MinusInfinity);
      fprintf(stderr, " MaxF = %18.8f\n", MaxFloat);
    }

    time_func("empty loop",     do_empty,          NTIMES);
    time_func("conversion",     do_conversion,     NTIMES);
    time_func("single add",     do_single_add,     NTIMES);
    time_func("single mul",     do_single_mul,     NTIMES);
    time_func("single div",     do_single_div,     NTIMES);
    time_func("double add",     do_double_add,     NTIMES);
    time_func("double mul",     do_double_mul,     NTIMES);
    time_func("double div",     do_double_div,     NTIMES);
    time_func("ROUND_UP/DOWN",  do_my_set_round,   NTIMES);
    time_func("sys set_round",  do_sys_set_round,  NTIMES);
    time_func("flt_mul",        do_flt_mul,        NTIMES);
    time_func("flt_mix",        do_flt_mix,        NTIMES);
    time_func("FMIN/FMAX",      do_FMIN_FMAX,      NTIMES);
    return(0);
  }
    
int do_empty(int ntimes)
  { int i;
    for (i=0; i<ntimes; i++) { /* do nothing */ }
    return (ntimes);
  }
 
int do_conversion(int ntimes)
  { int i;
    float x, y;
    double xx, yy;
    x = 0.1;
    xx = 0.7;
    for (i=0; i<ntimes; i++)
      { y = (float) xx; yy = (double) x;
        y = (float) xx; yy = (double) x;
        y = (float) xx; yy = (double) x;
        y = (float) xx; yy = (double) x;
        y = (float) xx; yy = (double) x;
        y = (float) xx; yy = (double) x;
        y = (float) xx; yy = (double) x;
        y = (float) xx; yy = (double) x;
        y = (float) xx; yy = (double) x;
        y = (float) xx; yy = (double) x;
      }
    return (10*ntimes);
  }
    
int do_single_add(int ntimes)
  { int i;
    float x, y;
    x = 0.1;
    y = 0.1;
    for (i=0; i<ntimes; i++)
      { x = x + y;
        x = x + y;
        x = x + y;
        x = x + y;
        x = x + y;
        x = x + y;
        x = x + y;
        x = x + y;
        x = x + y;
        x = x + y;
      }
    return (10*ntimes);
  }
    
int do_single_mul(int ntimes)
  { int i;
    float x, y1, y2;
    x = 0.1;
    y1 = 1.001;
    y2 = 1.0/y1;
    for (i=0; i<ntimes; i++)
      { x = x * y1;
        x = x * y2;
        x = x * y1;
        x = x * y2;
        x = x * y1;
        x = x * y2;
        x = x * y1;
        x = x * y2;
        x = x * y1;
        x = x * y2;
      }
    return (10*ntimes);
  }
    
int do_single_div(int ntimes)
  { int i;
    float x, y1, y2;
    x = 0.1;
    y1 = 1.001;
    y2 = 1.0/y1;
    for (i=0; i<ntimes; i++)
      { x = x / y1;
        x = x / y2;
        x = x / y1;
        x = x / y2;
        x = x / y1;
        x = x / y2;
        x = x / y1;
        x = x / y2;
        x = x / y1;
        x = x / y2;
      }
    return (10*ntimes);
  }
    
int do_double_add(int ntimes)
  { int i;
    double x, y;
    x = 0.1;
    y = 0.1;
    for (i=0; i<ntimes; i++)
      { x = x + y;
        x = x + y;
        x = x + y;
        x = x + y;
        x = x + y;
        x = x + y;
        x = x + y;
        x = x + y;
        x = x + y;
        x = x + y;
      }
    return (10*ntimes);
  }
    
int do_double_mul(int ntimes)
  { int i;
    double x, y1, y2;
    x = 0.1;
    y1 = 1.001;
    y2 = 1.0/y1;
    for (i=0; i<ntimes; i++)
      { x = x * y1;
        x = x * y2;
        x = x * y1;
        x = x * y2;
        x = x * y1;
        x = x * y2;
        x = x * y1;
        x = x * y2;
        x = x * y1;
        x = x * y2;
      }
    return (10*ntimes);
  }
    
int do_double_div(int ntimes)
  { int i;
    double x, y1, y2;
    x = 0.1;
    y1 = 1.001;
    y2 = 1.0/y1;
    for (i=0; i<ntimes; i++)
      { x = x / y1;
        x = x / y2;
        x = x / y1;
        x = x / y2;
        x = x / y1;
        x = x / y2;
        x = x / y1;
        x = x / y2;
        x = x / y1;
        x = x / y2;
      }
    return (10*ntimes);
  }
    
int do_my_set_round(int ntimes)
  { int i;
    for (i=0; i<ntimes; i++)
      { ROUND_UP;
        ROUND_DOWN;
        ROUND_UP;
        ROUND_DOWN;
        ROUND_UP;
        ROUND_DOWN;
        ROUND_UP;
        ROUND_DOWN;
        ROUND_UP;
        ROUND_DOWN;
      }
    return (10*ntimes);
  }

// #if (defined(SunOS4))
// int do_sys_set_round(int ntimes)
//   { int i;
//     char *out_dir;
//     for (i=0; i<ntimes; i++)
//       { 
//         ieee_flags("set", "direction", "positive", &out_dir);
//         ieee_flags("set", "direction", "negative", &out_dir);
// 
//         ieee_flags("set", "direction", "positive", &out_dir);
//         ieee_flags("set", "direction", "negative", &out_dir);
// 
//         ieee_flags("set", "direction", "positive", &out_dir);
//         ieee_flags("set", "direction", "negative", &out_dir);
// 
//         ieee_flags("set", "direction", "positive", &out_dir);
//         ieee_flags("set", "direction", "negative", &out_dir);
//       }
//     return (8*ntimes);
//   }
// #endif
// 
// #if (defined(SunOS5))
// int do_sys_set_round(int ntimes)
//   { int i;
//     fp_rnd out_dir;
//     for (i=0; i<ntimes; i++)
//       { 
//         out_dir = fpsetround(FP_RP);
//         out_dir = fpsetround(FP_RN);
//         out_dir = fpsetround(FP_RZ);
//         out_dir = fpsetround(FP_RM);
// 
//         out_dir = fpsetround(FP_RP);
//         out_dir = fpsetround(FP_RN);
//         out_dir = fpsetround(FP_RZ);
//         out_dir = fpsetround(FP_RM);
//       }
//     return (8*ntimes);
//   }
// #endif

int do_sys_set_round(int ntimes)
  { int i;
    int out_dir;
    for (i=0; i<ntimes; i++)
      { 
        out_dir = fesetround(FE_UPWARD);
        out_dir = fesetround(FE_DOWNWARD);
        out_dir = fesetround(FE_UPWARD);
        out_dir = fesetround(FE_DOWNWARD);
        out_dir = fesetround(FE_UPWARD);
        out_dir = fesetround(FE_DOWNWARD);
        out_dir = fesetround(FE_UPWARD);
        out_dir = fesetround(FE_DOWNWARD);
      }
    return (8*ntimes);
  }

int do_flt_mul(int ntimes)
  { int i;
    Float x, y1, y2, err;
    x = 0.1;
    y1 = 1.001;
    y2 = 1.0/y1;
    err = 0.0;
    for (i=0; i<ntimes; i++)
      { flt_mul(x, y1, &x, &err);
      }
    return (ntimes);
  }

int do_flt_mix(int ntimes)
  { int i;
    Float x, y1, y2, err;
    x = 0.1;
    y1 = 1.001;
    y2 = 1.0/y1;
    err = 0.0;
    for (i=0; i<ntimes; i++)
      { flt_mix(x, y2, y1, x, y2, &x, &err);
      }
    return (ntimes);
  }

int do_FMIN_FMAX(int ntimes)
  { int i;
    Float x, y1, y2, err;
    x = 1.0;
    y1 = 1.00001;
    y2 = 1.0/y1;
    err = 0.0;
    for (i=0; i<ntimes; i++)
      { x = FMIN(y1, y2);
        x = FMAX(y1, y2);
        x = FMIN(y1, y2);
        x = FMAX(y1, y2);
        x = FMIN(y1, y2);
        x = FMAX(y1, y2);
        x = FMIN(y1, y2);
        x = FMAX(y1, y2);
        x = FMIN(y1, y2);
        x = FMAX(y1, y2);
      }
    return (10*ntimes);
  }

