/* See rmxn_extra.h. */
/* Last edited on 2007-02-09 16:21:12 by stolfi */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include <rn.h>
#include <rmxn.h>
#include <jsrandom.h>
#include <affirm.h>

#include <rmxn_extra.h>

void rmxn_rot2(double c, double s, double *x, double *y);
  /* Rotates the vector {(x,y)} by an angle {theta}, given
     {x}, {y}, {c = cos(theta)}, and {s = sin(theta)}. */

void rmxn_full_simplex(int n, double V[])
  { int i, j;
    /* Set the last row to the properly scaled all-ones vector: */
    double rs = 1.0/sqrt(n);
    for (j = 0; j < n; j++) { V[n*n + j] = - rs; }
    /* Set the first {n} rows of {V} to scaled and shifted canonical {n-1}-simplex: */
    double d = rs/n;
    double u = (sqrt(1 + (n-1)*n*n) - 1)*d;
    for (i = 0; i < n; i++) 
      { for (j = 0; j < n; j++)
          { V[i*n + j] = (i == j ? u : 0) + d; }
      }
  }

void rmxn_rot2(double c, double s, double *x, double *y)
  { double X = c*(*x) - s*(*y);
    double Y = s*(*x) + c*(*y);
    (*x) = X; (*y) = Y;
  }

void rmxn_spin_rows(int m, int n, double A[], double R[])
  { /* Apply {n} two-axis rotations by random angles. */
    /* This is almost certainly not a uniformly random rotation. */
    if (R != A) { rmxn_copy(m, n, A, R); }
    int i, j0;
    for (j0 = 0; j0 < n; j0++)
      { int j1 = (j0+1)%n;
        double theta = 2*M_PI*drandom();
        double c = cos(theta);
        double s = sin(theta);
        for (i = 0; i < m; i++)
          { rmxn_rot2(c, s, &(R[i*n + j0]), &(R[i*n + j1])); }
      }
  }

void rmxn_spin_cols(int m, int n, double A[], double R[])
  { /* Apply {m} two-axis rotations by random angles. */
    /* This is almost certainly not a uniformly random rotation. */
    if (R != A) { rmxn_copy(m, n, A, R); }
    int i0, j;
    for (i0 = 0; i0 < m; i0++)
      { int i1 = (i0+1)%n;
        double theta = 2*M_PI*drandom();
        double c = cos(theta);
        double s = sin(theta);
        for (j = 0; j < n; j++)
          { rmxn_rot2(c, s, &(R[i0*n + j]), &(R[i1*n + j])); }
      }
  }
  
void rmxn_shift_rows(int m, int n, double A[], double v[], double R[])
  { int i, j;
    for (i = 0; i < m; i++)
      { for (j = 0; j < n; j++) 
          { int ij = i*n + j; R[ij] = A[ij] + v[j]; }
      }
  }
  
void rmxn_shift_cols(int m, int n, double v[], double A[], double R[])
  { int i, j;
    for (i = 0; i < m; i++)
      { for (j = 0; j < n; j++) 
          { int ij = i*n + j; R[ij] = A[ij] + v[i]; }
      }
  }

void rmxn_copy(int m, int n, double A[], double R[])
  { int mn = m*n;
    int ij;
    for (ij = 0; ij < mn; ij++) { R[ij] = A[ij]; }
  }
