/* rmxn_extra.h --- additional operation on MxN matrices */
/* Last edited on 2007-02-09 16:21:17 by stolfi */

#ifndef rmxn_extra_H
#define rmxn_extra_H

#include <stdio.h>
#include <rn.h>
#include <rmxn.h>

void rmxn_full_simplex(int n, double V[]);
  /* Stores into {V} the vertices of an {n}-dimensional simplex in
    {R^n} with center at the origin and unit radius. 
    
    The array {V} should have {(n+1)*n} elements, conceptually {n+1}
    rows and {n} columns. Element {V[i*n + j]} will be coordinate {j}
    of vertex {i}, for {i} in {0..n} and {j} in {0..n-1}. Vertices
    {0..n-1} will be in the positive orthant, close to the respective
    coordinate axes; and vertex {n} is {(-1,...,-1)/sqrt(n)}. */

void rmxn_spin_rows(int m, int n, double A[], double R[]);
  /* Applies a random rotation to each row of {A}, which is assumed to
    have {m} rows and {n} columns. Equivalent to computing {R=A*N}
    where {N} is a random orthonormal {nn} matrix. */

void rmxn_spin_cols(int m, int n, double A[], double R[]);
  /* Applies a random rotation to each column {A}, which is assumed to
    have {m} rows and {n} columns. Equivalent to computing {R=N*A}
    where {N} is a random orthonormal {mm} matrix. */

void rmxn_shift_rows(int m, int n, double A[], double v[], double R[]);
  /* Adds the vector {v[0..n-1]} to each row of the matrix {A},
    yielding matrix {R}. Both matrices are assumed to have {m} rows
    and {n} columns. */

void rmxn_shift_cols(int m, int n, double A[], double v[], double R[]);
  /* Adds the vector {v[0..m-1]} to each column of the matrix {A},
    yielding matrix {R}. Both matrices are assumed to have {m} rows
    and {n} columns. */

void rmxn_copy(int m, int n, double A[], double R[]);
  /* Copies the matrix {A} into {R}. Both matrices are assumed to have
    {m} rows and {n} columns. */

#endif

