#define PROG_NAME "testenumorb"
#define PROG_DESC "tests the {enum_orbits.h} procedures"
#define PROG_VERS "1.1"

/* Last edited on 2007-02-06 21:52:51 by stolfi */
/* Created on 2007-01-31 by J. Stolfi, UNICAMP */

#define PROG_COPYRIGHT \
  "Copyright  2007  by the State University of Campinas (UNICAMP)"

#include <enum_orbits.h>

#include <vec.h>
#include <bool.h>
#include <ref.h>
#include <affirm.h>
#include <jsrandom.h>
#include <jsmath.h>
#include <jstime.h>

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <values.h>

int main (int argc, char **argv);

void test_enor_procs(void);
  /* Tests the {enum_orbits.h} operations. */

int main (int argc, char **argv)
  { test_enor_procs();
    return 0;
  }
  
#define MAX_ELEM_BYTES 10000000
   /* Maximum memory to allocate for tests records. */
  
void test_enor_procs(void)
  { 
    uint ns = 4;
    enum_step_t stp[4];
    enum_visit_t vis[5];
    
    fprintf(stderr, "============================================================\n");
    fprintf(stderr, "testing with %d step functions\n", ns);
    
    auto ref_t step0(ref_t p);
    auto ref_t step1(ref_t p);
    auto ref_t step2(ref_t p);
    auto ref_t step3(ref_t p);
    auto ref_t gen_step(ref_t p, uint m, uint d);
    
    auto bool_t visit0(ref_t p);
    auto bool_t visit1(ref_t p);
    auto bool_t visit2(ref_t p);
    auto bool_t visit3(ref_t p);
    auto bool_t visit4(ref_t p);
    auto bool_t gen_visit(ref_t p, uint k);
    
    vis[0] = &visit0; stp[0] = &step0;
    vis[1] = &visit1; stp[1] = &step1;
    vis[2] = &visit2; stp[2] = &step2;
    vis[3] = &visit3; stp[3] = &step3;
    vis[4] = &visit4;
    
    /* The enumeration items will be addresses into the array {dom}: */
    uint dom[96];
    
    /* Fill {dom} with recognizable stuff: */
    { int i,j,k,r,s; 
      int m = 0;
      for (i = 0; i < 4; i++)
        { for (j = 0; j < 2; j++)
            { for (k = 0; k < 3; k++)
                { for (r = 0; r < 2; r++)
                    { for (s = 0; s < 2; s++)
                        { dom[m] = s + 10*(r + 10*(k + 10*(j + 10*i)));
                          m++;
                        }
                    }
                }
            }
            
        }
      assert(m == 96);
    }
    
    /* The root set is {r[0..2]}: */
    ref_t r[3]; 
    r[0] = &(dom[ 3]); 
    r[1] = &(dom[11]); 
    r[2] = &(dom[79]);
        
    ref_vec_t v = ref_vec_new(0); /* The list of visited items. */
        
    auto void init_visit_list(ref_vec_t *vP);
    /* Initializes the visit list {vP} to a single ref to the letter '*' */
    
    auto void print_visit_list(ref_vec_t *vP);
    /* Prints the visit list {vP} to stderr. */

    int i;
    
    for (i = 0; i < ns; i++)
      { uint *r = &(dom[7]);
        fprintf(stderr, "testing enum_cycle with %c, stp[%d], vis[%d]\n", *r, i, i);
        init_visit_list(&v);
        enum_cycle((ref_t)r, stp[i], vis[i], &v);
        print_visit_list(&v);
      }
    
    fprintf(stderr, "testing enum_orbits with stp[%d], vis[%d]\n", i, i);
    init_visit_list(&v);
    enum_orbits( ref_vec_desc(r,3), ns, stp, vis, &v);
    print_visit_list(&v);
    
    fprintf(stderr, "============================================================\n");
    return;

    /* Step procedures (perms of the addresses {&(dom[0..95])}): */
   
    ref_t step0(ref_t p)
      { return gen_step(p, 2, 1); }

    ref_t step1(ref_t p)
      { return gen_step(p, 4, 2); }

    ref_t step2(ref_t p)
      { return gen_step(p, 3, 1); }

    ref_t step3(ref_t p)
      { return gen_step(p, 8, 5); }

    ref_t gen_step(ref_t p, uint m, uint d)
      { /* Cycles by +6 each group of 12 items: */
        uint *q = (uint *)p;
        int i = q - dom;
        i = (i / m) * m + ((i + d) % m); 
        return (ref_t)(dom + i);
      }

    /* Visit procedures (print the {dom} entry addressed by {p}): */
   
    bool_t visit0(ref_t p)
      { return gen_visit(p, 0); }
      
    bool_t visit1(ref_t p)
      { return gen_visit(p, 1); }

    bool_t visit2(ref_t p)
      { return gen_visit(p, 2); }

    bool_t visit3(ref_t p)
      { return gen_visit(p, 3); }

    bool_t visit4(ref_t p)
      { return gen_visit(p, 4); } 
    
    bool_t gen_visit(ref_t p, uint k)
      { fprintf(stderr, "  visit%u(%05u)\n", k, *((uint *)p));
        return FALSE;
      }
    
    void init_visit_list(ref_vec_t *vP)
      { ref_vec_trim(vP, 0);
        ref_vec_trim(vP, 1);
        vP->el[0] = &(dom[26]); 
      }

    void print_visit_list(ref_vec_t *vP)
      { fprintf(stderr, "v = [");
        int j;
        for (j = 0; j < v.nel; j++)
          { fprintf(stderr, " %05u", *((uint *)(v.el[j]))); }
        fprintf(stderr, "]\n");
      }

  }
